/*
 * File Name : util.c
 *
 * Description : This file contains utility functions as followings:
 *               err_dump(),
 *               err_sys(),
 *	 	 err_ret(),
 *		 gethostaddr(),
 *		 pri_inet(),
 *		 getfilename()
 *
 * Programmed by : IlKyeun Ra, Syracuse University
 *           
*/
  
#include        <stdio.h>
#include        <sys/types.h>
#include        <sys/socket.h>
#include        <netinet/in.h>
#include        <arpa/inet.h>
#include	<netdb.h>
#include        <sys/param.h>
#include	"dvm.h"
#include	"vdce.h"


extern	struct	task	*_dvminfo;
extern	int		_total_data;
extern	int		_myvdcetaskid;
extern	char		*_myvdcetaskname;
extern	FILE		*jfp;

err_dump(msg)
char *msg;
{
	fprintf(jfp,"err:%s \n",msg);
        fflush(jfp);
	abort();	/* ask core dump */
}

err_sys(msg)
char *msg;
{
        fprintf(jfp,"err:%s \n",msg);
        fflush(jfp);
        exit(1);
}

err_ret(msg)
char *msg;
{
        fprintf(jfp,"err:%s \n",msg);
        fflush(jfp);
        return(0);
}

/*
 * Function Name : gethostaddr()
 *
 * Description   : get internet address from hostname
 *
 * Input(s)      : None
 *
 * Output(s)     : Internet address(e.g. 128.230.32.2)
*/
char *gethostaddr()
{
  char *hname;
  char *haddr;
  int  hlen=100;
  struct hostent *he;

  hname = (char *) calloc(hlen,sizeof(char));
  if (gethostname(hname,hlen)!=0) 
      err_sys("VDCE_INIT: can't find host name");

  if ( (he = gethostbyname(hname)) == NULL)
      err_sys("VDCE_INIT: can`t get host address");

  if ( (haddr = (char *) calloc(hlen,sizeof(char))) == NULL)
	err_dump("VDCE_INIT: No more memory");

  switch ( he->h_addrtype) {
    case AF_INET:
       pr_inet(he->h_addr_list,he->h_length,haddr);
       break;
    default:
	err_ret("unknow address type");
        break;
  }
  
  return(haddr);
}
/*
 * Function Name : pr_inet()
 *
 * Description   : convert internet address to ascii format(user readable)
 *
 * Input(s)      : internet - not user readable format
 *                 length   - length of internet address
 * 
 * Output(s)     : User readable internet address - ascii format
*/
pr_inet(listptr,length,addr)
char **listptr;
int  length;
char *addr;
{

     struct in_addr *ptr;

     while ( (ptr = (struct in_addr *) *listptr++) != NULL) 
       sprintf(addr,"%s",inet_ntoa(*ptr));
}

/*
 * Function name : getfilename()
 *
 * Description   : This function is creating a temporary file name 
 *
 * Result        : 1 - Success, 0 - Error
 *                 
*/
getfilename(fname)
char *fname;
{
     char pathname[MAXPATHLEN];
   
     if ((char *)getwd(pathname) != NULL) 
         if ( (fname = tempnam(&pathname[0],"vdce")) != NULL)
	   return(1);
         else 
	   err_ret("VDCE: can't creat a tempoary file\n");
     else
        err_ret("VDCE: can't get current directory\n");

     return(0);
}

/*
 * Function name : vdce_get_port() 
 *
 * Description   : find an entry of taksinformation table using port id
 *
 * Result        : index of proper entry from taskinformation table
 */

vdce_get_port(port,flag)
VDCE_PORT port;
register int flag;
{
     int rc,i;

     i = 0;
     while (i < _total_data) {

      if (flag == VDCESEND) {
	/* check for sending port */
         if ( _dvminfo[i].s_pid == (int) port  && 
              _dvminfo[i].s_tkid == _myvdcetaskid){
		rc = i;
		return(rc);
         }
         i++;
     }
     else {
	  /* check for receving port */
	  if (_dvminfo[i].d_pid == (int) port && 
	      _dvminfo[i].d_tkid == _myvdcetaskid) {
		rc = i;
		return(rc);
         }
         i++;
     }
    }
    return(-1); /* can't find a port id */
}

/*
 * Function name : vdce_data_represent()
 *
 * Description   : find an entry of taksinformation table using port id
 *
 * Result        : index of proper entry from taskinformation table
 */

vdce_data_represent(port,flag)
VDCE_PORT port;
register int flag;
{
     int rc,i;

     i = 0;
     while (i < _total_data) {

      if (flag == OUTPUT) {
        /* check for sending port */
         if ( _dvminfo[i].s_pid == (int) port  &&
              _dvminfo[i].s_tkid == _myvdcetaskid){
                if (_dvminfo[i].s_data_rep == _dvminfo[i].d_data_rep)
                        return(0); /*same */
                else return(1); /*defferent */
         }
         i++;
     }
     else {  /*INPUT*/
          /* check for receving port */
          if (_dvminfo[i].d_pid == (int) port &&
              _dvminfo[i].d_tkid == _myvdcetaskid) {
		if (_dvminfo[i].d_data_rep == _dvminfo[i].s_data_rep)
			return(0); /*same */
                else return(1); /*defferent */
         }
         i++;
     }
    }
    return(-1); /* can't find a port id */
}

/*
 * Function Name : vdce_input_name()
 *
 * Description : find input filename for input access
 */
char* vdce_input_name(taskid, port)
int taskid;
int port;
{
	int  i;
	i = 0;
 	while (i < _total_data) {
	if( _dvminfo[i].d_tkid == taskid && 
	    _dvminfo[i].d_pid == port) {
	     return (_dvminfo[i].v_name);
	    }
	i++;
	}
}

/*
 * Function Name : vdce_output_name()
 * Description : find output filename for output access
 */
char* vdce_output_name(taskid, port)
int taskid;
int port;
{
	int rc, i;
	i = 0;
 	while (i < _total_data) {
	if( _dvminfo[i].s_tkid == taskid && 
	    _dvminfo[i].s_pid == port) {
	     return (_dvminfo[i].v_name);
            }
	i++;
	}
}

/*
 * Function Name : vdce_get_taskinfo
 *
 * Description   : providing task information upon request of ADL
 *
 * Input(s)      : 
 *
 * Output(s)     : task information
 *
 */
vdce_get_taskinfo()
{
    VDCE_TASKINFO	*mytaskinfo;
    register	int 	i;

   /* call vdce_def which is in user task to get task informaiton */
    mytaskinfo = (VDCE_TASKINFO *) malloc(sizeof(VDCE_TASKINFO));
    mytaskinfo->noofinputs = 0;
    mytaskinfo->noofoutputs = 0;

    vdce_def(mytaskinfo);

   /* print task name */
   fprintf(jfp,"Task name is = %s\n",mytaskinfo->taskname);

   /* print input names */
   fprintf(jfp,"No. of Inputs = %d\n",mytaskinfo->noofinputs);

   for(i=0;i<mytaskinfo->noofinputs;i++)
      fprintf(jfp,"input[%d]=%s\n",i+1,mytaskinfo->inputnames[i]);

   /* print outputs names */
   fprintf(jfp,"No. of Outputs = %d\n",mytaskinfo->noofoutputs);

   for(i=0;i<mytaskinfo->noofoutputs;i++)
      fprintf(jfp,"output[%d]=%s\n",i+1,mytaskinfo->outputnames[i]);

}
	

/*
 * Function Name : vdce_set_taskname
 *
 * Description   : define task name 
 *
 *
 */
vdce_set_taskname(mytask,taskname)
VDCE_TASKINFO *mytask;
char          *taskname;
{
	mytask->taskname = (char *) calloc(strlen(taskname)+4,sizeof(char));
	sprintf(mytask->taskname,"%s",taskname);
}
	
/*
 * Function name : vdce_set_inputs
 *
 * Description   : define environment of input arguments
 *
 */
vdce_set_inputs(mytask,names)
VDCE_TASKINFO	*mytask;
char		*names;
{
	mytask->inputnames[mytask->noofinputs] = 
			(char *) calloc(strlen(names)+4,sizeof(char));
	sprintf(mytask->inputnames[mytask->noofinputs],"%s",names);
	mytask->noofinputs ++;
}

/*
 * Function name : vdce_set_outputs
 *
 * Description   : define environment of output arguments
 *
 */
vdce_set_outputs(mytask,names)
VDCE_TASKINFO	*mytask;
char		*names;
{
	mytask->outputnames[mytask->noofoutputs] = 
		(char *) calloc(strlen(names)+4,sizeof(char));
	sprintf(mytask->outputnames[mytask->noofoutputs],"%s",names);
	mytask->noofoutputs++;
}

vdce_get_my_id()
{
	return(_myvdcetaskid);
}
char *vdce_get_my_name()
{
	return(_myvdcetaskname);
}

create_header_address(addr, userid, applid, taskid)
char *addr;
int userid, applid, taskid;
{
	char temp[10];

	if ((userid >= 0 ) && (userid < 10)){
		sprintf(temp, "000%d",userid);
		strcat(addr, temp);
		}
	else if ((userid >=10) && (userid < 100)){
		sprintf(temp, "00%d",userid);
		strcat(addr, temp);
		}
	else if ((userid >= 100) && (userid < 1000)) {
		sprintf(temp, "0%d",userid);
                strcat(addr, temp);
		}
	else {
		sprintf(temp, "%d",userid);
                strcat(addr, temp);
		}

        if ((applid >= 0 ) && (applid < 10)){
                sprintf(temp, "000%d",applid);
                strcat(addr, temp);
                }
        else if ((applid >=10) && (applid < 100)){
                sprintf(temp, "00%d",applid);
                strcat(addr, temp);
                }
        else if ((applid >= 100) && (applid < 1000)) {
                sprintf(temp, "0%d",applid);
                strcat(addr, temp);
                }
        else {
                sprintf(temp, "%d",applid);
                strcat(addr, temp);
                }
 
      if ((taskid >= 0 ) && (taskid < 10)){
                sprintf(temp, "00%d",taskid);
                strcat(addr, temp);
                }
        else if ((taskid >=10) && (taskid < 100)){
                sprintf(temp, "0%d",taskid);
                strcat(addr, temp);
                }
        else {
                sprintf(temp, "%d",taskid);
                strcat(addr, temp);
                }
 
#ifdef DEBUG
	fprintf(jfp, "addr = %s\n", addr);
	fflush(jfp);
#endif
}
